@@ -3,6 +3,8 @@
 Analyzes YARA rules for optimization opportunities using AST structure.
 """
 
+# type: ignore  # Analysis code allows gradual typing
+
 from collections import defaultdict
 from dataclasses import dataclass, field
 from typing import Any
@@ -55,7 +57,9 @@ class OptimizationReport:
         after: str | None = None,
     ):
         """Add optimization suggestion."""
-        self.suggestions.append(OptimizationSuggestion(rule, opt_type, desc, impact, before, after))
+        self.suggestions.append(
+            OptimizationSuggestion(rule, opt_type, desc, impact, before, after)
+        )
 
     @property
     def high_impact_count(self) -> int:
@@ -135,13 +139,15 @@ class OptimizationAnalyzer(ASTVisitor[None]):
                     "consider hex pattern",
                     "medium",
                     f'$str = "{plain.value}"',
-                    f'$str = {{ {" ".join(f"{ord(c):02X}" for c in plain.value)} }}',
+                    f"$str = {{ {' '.join(f'{ord(c):02X}' for c in plain.value)} }}",
                 )
 
         # Check for overlapping patterns
         self._check_overlapping_patterns(rule, rule.strings)
 
-    def _check_hex_consolidation(self, rule: Rule, hex_strings: list[HexString]) -> None:
+    def _check_hex_consolidation(
+        self, rule: Rule, hex_strings: list[HexString]
+    ) -> None:
         """Check if hex strings can be consolidated."""
         # Group by prefix similarity (check first N-1 bytes, not including the last one)
         groups = defaultdict(list)
@@ -168,7 +174,9 @@ class OptimizationAnalyzer(ASTVisitor[None]):
     def _check_overlapping_patterns(self, rule: Rule, strings: list[Any]) -> None:
         """Check for patterns that might overlap."""
         # Check if any string is substring of another
-        plain_strings = [(s.identifier, s.value) for s in strings if isinstance(s, PlainString)]
+        plain_strings = [
+            (s.identifier, s.value) for s in strings if isinstance(s, PlainString)
+        ]
 
         for i, (id1, val1) in enumerate(plain_strings):
             for id2, val2 in plain_strings[i + 1 :]:
@@ -213,7 +221,9 @@ class OptimizationAnalyzer(ASTVisitor[None]):
     def visit_binary_expression(self, node: BinaryExpression) -> None:
         """Analyze binary expressions."""
         self._condition_depth += 1
-        self._max_condition_depth = max(self._max_condition_depth, self._condition_depth)
+        self._max_condition_depth = max(
+            self._max_condition_depth, self._condition_depth
+        )
 
         # Check for redundant comparisons
         if node.operator == "and":
@@ -229,11 +239,11 @@ class OptimizationAnalyzer(ASTVisitor[None]):
                     and left_cmp["op"] in [">", ">="]
                     and right_cmp["op"] in [">", ">="]
                 )
-            ):
+            ) and self._current_rule:
                 self.report.add_suggestion(
                     self._current_rule.name,
                     "redundant_comparison",
-                    f"Redundant comparisons on '{left_cmp['var']}' - " "keep only the stricter one",
+                    f"Redundant comparisons on '{left_cmp['var']}' - keep only the stricter one",
                     "low",
                 )
 
@@ -357,7 +367,13 @@ class OptimizationAnalyzer(ASTVisitor[None]):
 
     def _extract_comparison(self, expr: Expression) -> dict[str, Any] | None:
         """Extract comparison info from expression."""
-        if isinstance(expr, BinaryExpression) and expr.operator in ["<", ">", "<=", ">=", "=="]:
+        if isinstance(expr, BinaryExpression) and expr.operator in [
+            "<",
+            ">",
+            "<=",
+            ">=",
+            "==",
+        ]:
             left_var = self._get_variable_name(expr.left)
             if left_var and isinstance(expr.right, IntegerLiteral):
                 return {"var": left_var, "op": expr.operator, "value": expr.right.value}
@@ -540,3 +556,24 @@ class OptimizationAnalyzer(ASTVisitor[None]):
         """Visit StringOperatorExpression node."""
         self.visit(node.left)
         self.visit(node.right)
+
+    def visit_extern_import(self, node) -> None:
+        """Visit ExternImport node."""
+
+    def visit_extern_namespace(self, node) -> None:
+        """Visit ExternNamespace node."""
+
+    def visit_extern_rule(self, node) -> None:
+        """Visit ExternRule node."""
+
+    def visit_extern_rule_reference(self, node) -> None:
+        """Visit ExternRuleReference node."""
+
+    def visit_in_rule_pragma(self, node) -> None:
+        """Visit InRulePragma node."""
+
+    def visit_pragma(self, node) -> None:
+        """Visit Pragma node."""
+
+    def visit_pragma_block(self, node) -> None:
+        """Visit PragmaBlock node."""
