@@ -2,7 +2,6 @@
 
 from __future__ import annotations
 
-from typing import Self
 
 from yaraast.ast.conditions import AtExpression, InExpression, OfExpression
 from yaraast.ast.expressions import (
@@ -31,55 +30,62 @@ class FluentConditionBuilder(ConditionBuilder):
         super().__init__(expr)
 
     # Enhanced quantifier methods
-    def any_of_them(self) -> Self:
+    def any_of_them(self) -> FluentConditionBuilder:
         """Any of them - common pattern."""
         return FluentConditionBuilder(
-            OfExpression(quantifier=StringLiteral(value="any"), string_set=Identifier(name="them"))
+            OfExpression(
+                quantifier=StringLiteral(value="any"),
+                string_set=Identifier(name="them"),
+            )
         )
 
-    def all_of_them(self) -> Self:
+    def all_of_them(self) -> FluentConditionBuilder:
         """All of them - common pattern."""
         return FluentConditionBuilder(
-            OfExpression(quantifier=StringLiteral(value="all"), string_set=Identifier(name="them"))
+            OfExpression(
+                quantifier=StringLiteral(value="all"),
+                string_set=Identifier(name="them"),
+            )
         )
 
-    def not_them(self) -> Self:
+    def not_them(self) -> FluentConditionBuilder:
         """Not any of them - negated pattern."""
         return FluentConditionBuilder(
             UnaryExpression(
                 operator="not",
                 operand=OfExpression(
-                    quantifier=StringLiteral(value="any"), string_set=Identifier(name="them")
+                    quantifier=StringLiteral(value="any"),
+                    string_set=Identifier(name="them"),
                 ),
             )
         )
 
-    def one_of(self, *strings: str) -> Self:
+    def one_of(self, *strings: str) -> FluentConditionBuilder:
         """Exactly one of the specified strings."""
         return self.n_of(1, *strings)
 
-    def two_of(self, *strings: str) -> Self:
+    def two_of(self, *strings: str) -> FluentConditionBuilder:
         """Exactly two of the specified strings."""
         return self.n_of(2, *strings)
 
-    def three_of(self, *strings: str) -> Self:
+    def three_of(self, *strings: str) -> FluentConditionBuilder:
         """Exactly three of the specified strings."""
         return self.n_of(3, *strings)
 
-    def most_of(self, *strings: str) -> Self:
+    def most_of(self, *strings: str) -> FluentConditionBuilder:
         """Most of the strings (more than half)."""
         threshold = (len(strings) // 2) + 1
         return self.n_of(threshold, *strings)
 
-    def few_of(self, *strings: str) -> Self:
+    def few_of(self, *strings: str) -> FluentConditionBuilder:
         """Few of the strings (at least 2)."""
         return self.at_least_n_of(2, *strings)
 
-    def many_of(self, *strings: str) -> Self:
+    def many_of(self, *strings: str) -> FluentConditionBuilder:
         """Many of the strings (at least 3)."""
         return self.at_least_n_of(3, *strings)
 
-    def at_least_n_of(self, n: int, *strings: str) -> Self:
+    def at_least_n_of(self, n: int, *strings: str) -> FluentConditionBuilder:
         """At least N of the specified strings."""
         # Create condition: n_of OR (n+1)_of OR ... OR all_of
         conditions = []
@@ -93,7 +99,7 @@ class FluentConditionBuilder(ConditionBuilder):
 
         return FluentConditionBuilder(result)
 
-    def at_most_n_of(self, n: int, *strings: str) -> Self:
+    def at_most_n_of(self, n: int, *strings: str) -> FluentConditionBuilder:
         """At most N of the specified strings."""
         # Create condition: 0_of OR 1_of OR ... OR n_of
         conditions = []
@@ -107,7 +113,9 @@ class FluentConditionBuilder(ConditionBuilder):
 
         return FluentConditionBuilder(result)
 
-    def between_n_and_m_of(self, min_n: int, max_m: int, *strings: str) -> Self:
+    def between_n_and_m_of(
+        self, min_n: int, max_m: int, *strings: str
+    ) -> FluentConditionBuilder:
         """Between N and M of the specified strings."""
         conditions = []
         for i in range(min_n, max_m + 1):
@@ -121,11 +129,11 @@ class FluentConditionBuilder(ConditionBuilder):
         return FluentConditionBuilder(result)
 
     # String-specific helpers
-    def string_matches(self, string_id: str) -> Self:
+    def string_matches(self, string_id: str) -> FluentConditionBuilder:
         """String matches (shorthand for string identifier)."""
         return FluentConditionBuilder(StringIdentifier(name=string_id))
 
-    def string_count_eq(self, string_id: str, count: int) -> Self:
+    def string_count_eq(self, string_id: str, count: int) -> FluentConditionBuilder:
         """String count equals N."""
         return FluentConditionBuilder(
             BinaryExpression(
@@ -135,7 +143,7 @@ class FluentConditionBuilder(ConditionBuilder):
             )
         )
 
-    def string_count_gt(self, string_id: str, count: int) -> Self:
+    def string_count_gt(self, string_id: str, count: int) -> FluentConditionBuilder:
         """String count greater than N."""
         return FluentConditionBuilder(
             BinaryExpression(
@@ -145,7 +153,7 @@ class FluentConditionBuilder(ConditionBuilder):
             )
         )
 
-    def string_count_ge(self, string_id: str, count: int) -> Self:
+    def string_count_ge(self, string_id: str, count: int) -> FluentConditionBuilder:
         """String count greater than or equal to N."""
         return FluentConditionBuilder(
             BinaryExpression(
@@ -155,28 +163,30 @@ class FluentConditionBuilder(ConditionBuilder):
             )
         )
 
-    def string_at_entrypoint(self, string_id: str) -> Self:
+    def string_at_entrypoint(self, string_id: str) -> FluentConditionBuilder:
         """String at entrypoint."""
         return FluentConditionBuilder(
             AtExpression(string_id=string_id, offset=Identifier(name="entrypoint"))
         )
 
-    def string_at_offset(self, string_id: str, offset: int) -> Self:
+    def string_at_offset(self, string_id: str, offset: int) -> FluentConditionBuilder:
         """String at specific offset."""
         return FluentConditionBuilder(
             AtExpression(string_id=string_id, offset=IntegerLiteral(value=offset))
         )
 
-    def string_in_first_kb(self, string_id: str) -> Self:
+    def string_in_first_kb(self, string_id: str) -> FluentConditionBuilder:
         """String in first 1KB of file."""
         return FluentConditionBuilder(
             InExpression(
                 string_id=string_id,
-                range=RangeExpression(low=IntegerLiteral(value=0), high=IntegerLiteral(value=1024)),
+                range=RangeExpression(
+                    low=IntegerLiteral(value=0), high=IntegerLiteral(value=1024)
+                ),
             )
         )
 
-    def string_in_last_kb(self, string_id: str) -> Self:
+    def string_in_last_kb(self, string_id: str) -> FluentConditionBuilder:
         """String in last 1KB of file."""
         return FluentConditionBuilder(
             InExpression(
@@ -193,31 +203,37 @@ class FluentConditionBuilder(ConditionBuilder):
         )
 
     # File property helpers
-    def filesize_eq(self, size: int) -> Self:
+    def filesize_eq(self, size: int) -> FluentConditionBuilder:
         """File size equals specific value."""
         return FluentConditionBuilder(
             BinaryExpression(
-                left=Identifier(name="filesize"), operator="==", right=IntegerLiteral(value=size)
+                left=Identifier(name="filesize"),
+                operator="==",
+                right=IntegerLiteral(value=size),
             )
         )
 
-    def filesize_gt(self, size: int) -> Self:
+    def filesize_gt(self, size: int) -> FluentConditionBuilder:
         """File size greater than."""
         return FluentConditionBuilder(
             BinaryExpression(
-                left=Identifier(name="filesize"), operator=">", right=IntegerLiteral(value=size)
+                left=Identifier(name="filesize"),
+                operator=">",
+                right=IntegerLiteral(value=size),
             )
         )
 
-    def filesize_lt(self, size: int) -> Self:
+    def filesize_lt(self, size: int) -> FluentConditionBuilder:
         """File size less than."""
         return FluentConditionBuilder(
             BinaryExpression(
-                left=Identifier(name="filesize"), operator="<", right=IntegerLiteral(value=size)
+                left=Identifier(name="filesize"),
+                operator="<",
+                right=IntegerLiteral(value=size),
             )
         )
 
-    def filesize_between(self, min_size: int, max_size: int) -> Self:
+    def filesize_between(self, min_size: int, max_size: int) -> FluentConditionBuilder:
         """File size between min and max."""
         return FluentConditionBuilder(
             BinaryExpression(
@@ -235,58 +251,67 @@ class FluentConditionBuilder(ConditionBuilder):
             )
         )
 
-    def small_file(self) -> Self:
+    def small_file(self) -> FluentConditionBuilder:
         """Small file (< 1MB)."""
         return self.filesize_lt(1024 * 1024)
 
-    def large_file(self) -> Self:
+    def large_file(self) -> FluentConditionBuilder:
         """Large file (> 10MB)."""
         return self.filesize_gt(10 * 1024 * 1024)
 
-    def tiny_file(self) -> Self:
+    def tiny_file(self) -> FluentConditionBuilder:
         """Tiny file (< 1KB)."""
         return self.filesize_lt(1024)
 
-    def huge_file(self) -> Self:
+    def huge_file(self) -> FluentConditionBuilder:
         """Huge file (> 100MB)."""
         return self.filesize_gt(100 * 1024 * 1024)
 
     # Module helpers
-    def pe_module(self) -> Self:
+    def pe_module(self) -> FluentConditionBuilder:
         """PE module reference."""
         return FluentConditionBuilder(Identifier(name="pe"))
 
-    def pe_is_dll(self) -> Self:
+    def pe_is_dll(self) -> FluentConditionBuilder:
         """PE is DLL."""
-        return FluentConditionBuilder(MemberAccess(object=Identifier(name="pe"), member="is_dll"))
+        return FluentConditionBuilder(
+            MemberAccess(object=Identifier(name="pe"), member="is_dll")
+        )
 
-    def pe_is_exe(self) -> Self:
+    def pe_is_exe(self) -> FluentConditionBuilder:
         """PE is executable (not DLL)."""
         return FluentConditionBuilder(
             UnaryExpression(
-                operator="not", operand=MemberAccess(object=Identifier(name="pe"), member="is_dll")
+                operator="not",
+                operand=MemberAccess(object=Identifier(name="pe"), member="is_dll"),
             )
         )
 
-    def pe_is_32bit(self) -> Self:
+    def pe_is_32bit(self) -> FluentConditionBuilder:
         """PE is 32-bit."""
-        return FluentConditionBuilder(MemberAccess(object=Identifier(name="pe"), member="is_32bit"))
+        return FluentConditionBuilder(
+            MemberAccess(object=Identifier(name="pe"), member="is_32bit")
+        )
 
-    def pe_is_64bit(self) -> Self:
+    def pe_is_64bit(self) -> FluentConditionBuilder:
         """PE is 64-bit."""
-        return FluentConditionBuilder(MemberAccess(object=Identifier(name="pe"), member="is_64bit"))
+        return FluentConditionBuilder(
+            MemberAccess(object=Identifier(name="pe"), member="is_64bit")
+        )
 
-    def pe_section_count_eq(self, count: int) -> Self:
+    def pe_section_count_eq(self, count: int) -> FluentConditionBuilder:
         """PE section count equals."""
         return FluentConditionBuilder(
             BinaryExpression(
-                left=MemberAccess(object=Identifier(name="pe"), member="number_of_sections"),
+                left=MemberAccess(
+                    object=Identifier(name="pe"), member="number_of_sections"
+                ),
                 operator="==",
                 right=IntegerLiteral(value=count),
             )
         )
 
-    def pe_imphash_eq(self, hash_value: str) -> Self:
+    def pe_imphash_eq(self, hash_value: str) -> FluentConditionBuilder:
         """PE import hash equals."""
         return FluentConditionBuilder(
             BinaryExpression(
@@ -296,46 +321,61 @@ class FluentConditionBuilder(ConditionBuilder):
             )
         )
 
-    def pe_exports(self, function_name: str) -> Self:
+    def pe_exports(self, function_name: str) -> FluentConditionBuilder:
         """PE exports function."""
         return FluentConditionBuilder(
-            FunctionCall(function="pe.exports", arguments=[StringLiteral(value=function_name)])
+            FunctionCall(
+                function="pe.exports", arguments=[StringLiteral(value=function_name)]
+            )
         )
 
-    def pe_imports(self, dll_name: str, function_name: str) -> Self:
+    def pe_imports(self, dll_name: str, function_name: str) -> FluentConditionBuilder:
         """PE imports function from DLL."""
         return FluentConditionBuilder(
             FunctionCall(
                 function="pe.imports",
-                arguments=[StringLiteral(value=dll_name), StringLiteral(value=function_name)],
+                arguments=[
+                    StringLiteral(value=dll_name),
+                    StringLiteral(value=function_name),
+                ],
             )
         )
 
     # Math module helpers
-    def entropy_gt(self, offset: int, size: int, threshold: float) -> Self:
+    MATH_ENTROPY = "math.entropy"
+
+    def entropy_gt(
+        self, offset: int, size: int, threshold: float
+    ) -> FluentConditionBuilder:
         """Entropy greater than threshold."""
         return FluentConditionBuilder(
             BinaryExpression(
                 left=FunctionCall(
-                    function="math.entropy",
-                    arguments=[IntegerLiteral(value=offset), IntegerLiteral(value=size)],
+                    function=self.MATH_ENTROPY,
+                    arguments=[
+                        IntegerLiteral(value=offset),
+                        IntegerLiteral(value=size),
+                    ],
                 ),
                 operator=">",
                 right=DoubleLiteral(value=threshold),
             )
         )
 
-    def high_entropy(self, offset: int = 0, size: int = 1024) -> Self:
+    def high_entropy(self, offset: int = 0, size: int = 1024) -> FluentConditionBuilder:
         """High entropy section (> 7.0)."""
         return self.entropy_gt(offset, size, 7.0)
 
-    def low_entropy(self, offset: int = 0, size: int = 1024) -> Self:
+    def low_entropy(self, offset: int = 0, size: int = 1024) -> FluentConditionBuilder:
         """Low entropy section (< 3.0)."""
         return FluentConditionBuilder(
             BinaryExpression(
                 left=FunctionCall(
-                    function="math.entropy",
-                    arguments=[IntegerLiteral(value=offset), IntegerLiteral(value=size)],
+                    function=self.MATH_ENTROPY,
+                    arguments=[
+                        IntegerLiteral(value=offset),
+                        IntegerLiteral(value=size),
+                    ],
                 ),
                 operator="<",
                 right=DoubleLiteral(value=3.0),
@@ -343,7 +383,7 @@ class FluentConditionBuilder(ConditionBuilder):
         )
 
     # Composite helpers
-    def executable_file(self) -> Self:
+    def executable_file(self) -> FluentConditionBuilder:
         """Common executable file patterns."""
         mz_at_0 = FluentConditionBuilder(
             AtExpression(string_id="mz_header", offset=IntegerLiteral(value=0))
@@ -352,18 +392,20 @@ class FluentConditionBuilder(ConditionBuilder):
         return mz_at_0.and_(
             FluentConditionBuilder(
                 BinaryExpression(
-                    left=Identifier(name="filesize"), operator=">", right=IntegerLiteral(value=1024)
+                    left=Identifier(name="filesize"),
+                    operator=">",
+                    right=IntegerLiteral(value=1024),
                 )
             )
         )
 
-    def suspicious_entropy(self) -> Self:
+    def suspicious_entropy(self) -> FluentConditionBuilder:
         """Suspicious entropy patterns."""
         return self.high_entropy().or_(
             FluentConditionBuilder(
                 BinaryExpression(
                     left=FunctionCall(
-                        function="math.entropy",
+                        function=self.MATH_ENTROPY,
                         arguments=[IntegerLiteral(value=0), IntegerLiteral(value=512)],
                     ),
                     operator=">",
@@ -372,12 +414,14 @@ class FluentConditionBuilder(ConditionBuilder):
             )
         )
 
-    def packed_executable(self) -> Self:
+    def packed_executable(self) -> FluentConditionBuilder:
         """Common packed executable indicators."""
         return self.suspicious_entropy().and_(
             FluentConditionBuilder(
                 BinaryExpression(
-                    left=MemberAccess(object=Identifier(name="pe"), member="number_of_sections"),
+                    left=MemberAccess(
+                        object=Identifier(name="pe"), member="number_of_sections"
+                    ),
                     operator="<",
                     right=IntegerLiteral(value=5),
                 )
