@@ -35,7 +35,7 @@ class EvaluationContext:
 class YaraEvaluator(ASTVisitor[Any]):
     """Evaluate YARA conditions against byte data."""
 
-    def __init__(self, data: bytes, modules: MockModuleRegistry | None = None):
+    def __init__(self, data: bytes = b"", modules: MockModuleRegistry | None = None):
         self.data = data
         self.context = EvaluationContext(data=data)
         self.string_matcher = StringMatcher()
@@ -84,7 +84,7 @@ class YaraEvaluator(ASTVisitor[Any]):
         return node.value
 
     def visit_integer_literal(self, node: IntegerLiteral) -> int:
-        return node.value
+        return int(node.value)
 
     def visit_double_literal(self, node: DoubleLiteral) -> float:
         return node.value
@@ -124,19 +124,31 @@ class YaraEvaluator(ASTVisitor[Any]):
 
     def visit_string_count(self, node: StringCount) -> int:
         """Get count of string matches."""
-        string_id = f"${node.string_id}" if not node.string_id.startswith("$") else node.string_id
+        string_id = (
+            f"${node.string_id}"
+            if not node.string_id.startswith("$")
+            else node.string_id
+        )
         return self.string_matcher.get_match_count(string_id)
 
     def visit_string_offset(self, node: StringOffset) -> int:
         """Get offset of string match."""
-        string_id = f"${node.string_id}" if not node.string_id.startswith("$") else node.string_id
+        string_id = (
+            f"${node.string_id}"
+            if not node.string_id.startswith("$")
+            else node.string_id
+        )
         index = self.visit(node.index) if node.index else 0
         offset = self.string_matcher.get_match_offset(string_id, index)
         return offset if offset is not None else -1
 
     def visit_string_length(self, node: StringLength) -> int:
         """Get length of string match."""
-        string_id = f"${node.string_id}" if not node.string_id.startswith("$") else node.string_id
+        string_id = (
+            f"${node.string_id}"
+            if not node.string_id.startswith("$")
+            else node.string_id
+        )
         index = self.visit(node.index) if node.index else 0
         length = self.string_matcher.get_match_length(string_id, index)
         return length if length is not None else 0
@@ -328,7 +340,9 @@ class YaraEvaluator(ASTVisitor[Any]):
         """Evaluate 'in' expression."""
         range_val = self.visit(node.range)
         if isinstance(range_val, range):
-            return self.string_matcher.string_in(node.string_id, range_val.start, range_val.stop)
+            return self.string_matcher.string_in(
+                node.string_id, range_val.start, range_val.stop
+            )
         return False
 
     def visit_of_expression(self, node: OfExpression) -> bool:
@@ -603,3 +617,35 @@ class YaraEvaluator(ASTVisitor[Any]):
         # This is handled in binary_expression for string operators
         # But we can add specific handling here if needed
         return self.visit_binary_expression(node)
+
+    def visit_extern_import(self, node):
+        """Visit extern import - not evaluated."""
+        pass
+
+    def visit_extern_namespace(self, node):
+        """Visit extern namespace - not evaluated."""
+        pass
+
+    def visit_extern_rule(self, node):
+        """Visit extern rule - not evaluated."""
+        pass
+
+    def visit_extern_rule_reference(self, node):
+        """Visit extern rule reference - not evaluated."""
+        pass
+
+    def visit_in_rule_pragma(self, node):
+        """Visit in-rule pragma - not evaluated."""
+        pass
+
+    def visit_pragma(self, node):
+        """Visit pragma - not evaluated."""
+        pass
+
+    def visit_pragma_block(self, node):
+        """Visit pragma block - not evaluated."""
+        pass
+
+
+# Alias for compatibility
+Evaluator = YaraEvaluator
