@@ -10,7 +10,7 @@ from jinja2 import Template
 from yaraast.visitor import ASTVisitor
 
 if TYPE_CHECKING:
-    from yaraast.ast.base import YaraFile
+    from yaraast.ast.base import ASTNode, YaraFile
     from yaraast.ast.rules import Rule
     from yaraast.ast.strings import HexString, PlainString, RegexString
 
@@ -18,12 +18,15 @@ if TYPE_CHECKING:
 class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
     """Generates HTML collapsible tree visualization from YARA AST."""
 
-    def __init__(self, include_metadata: bool = True):
+    def __init__(self, include_metadata: bool = True) -> None:
         self.include_metadata = include_metadata
         self.node_counter = 0
 
     def generate_html(
-        self, ast: YaraFile, output_path: str | None = None, title: str = "YARA AST Visualization"
+        self,
+        ast: YaraFile,
+        output_path: str | None = None,
+        title: str = "YARA AST Visualization",
     ) -> str:
         """Generate HTML tree visualization."""
         self.node_counter = 0
@@ -38,7 +41,10 @@ class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
         return html_content
 
     def generate_interactive_html(
-        self, ast: YaraFile, output_path: str | None = None, title: str = "Interactive YARA AST"
+        self,
+        ast: YaraFile,
+        output_path: str | None = None,
+        title: str = "Interactive YARA AST",
     ) -> str:
         """Generate interactive HTML with search and filtering."""
         self.node_counter = 0
@@ -269,10 +275,15 @@ class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
         stats = self._calculate_stats(tree_data) if self.include_metadata else None
 
         return template.render(
-            title=title, tree_data=tree_data, stats=stats, render_node=self._create_render_macro()
+            title=title,
+            tree_data=tree_data,
+            stats=stats,
+            render_node=self._create_render_macro(),
         )
 
-    def _render_interactive_template(self, tree_data: dict[str, Any], title: str) -> str:
+    def _render_interactive_template(
+        self, tree_data: dict[str, Any], title: str
+    ) -> str:
         """Render interactive HTML template with search and filtering."""
         template = Template(
             """
@@ -927,22 +938,46 @@ class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
 
     # Required visitor methods (minimal implementations)
     def visit_string_definition(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "String Definition", "node_class": "string"}
+        return {
+            "id": self._get_node_id(),
+            "label": "String Definition",
+            "node_class": "string",
+        }
 
     def visit_hex_token(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Hex Token", "node_class": "hex-token"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Hex Token",
+            "node_class": "hex-token",
+        }
 
     def visit_hex_jump(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Hex Jump", "node_class": "hex-jump"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Hex Jump",
+            "node_class": "hex-jump",
+        }
 
     def visit_hex_alternative(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Hex Alternative", "node_class": "hex-alt"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Hex Alternative",
+            "node_class": "hex-alt",
+        }
 
     def visit_hex_nibble(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Hex Nibble", "node_class": "hex-nibble"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Hex Nibble",
+            "node_class": "hex-nibble",
+        }
 
     def visit_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Expression",
+            "node_class": "expression",
+        }
 
     def visit_identifier(self, node) -> dict[str, Any]:
         return {
@@ -953,25 +988,53 @@ class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
         }
 
     def visit_string_count(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "String Count", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "String Count",
+            "node_class": "expression",
+        }
 
     def visit_string_offset(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "String Offset", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "String Offset",
+            "node_class": "expression",
+        }
 
     def visit_string_length(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "String Length", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "String Length",
+            "node_class": "expression",
+        }
 
     def visit_double_literal(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Double Literal", "node_class": "literal"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Double Literal",
+            "node_class": "literal",
+        }
 
     def visit_string_literal(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "String Literal", "node_class": "literal"}
+        return {
+            "id": self._get_node_id(),
+            "label": "String Literal",
+            "node_class": "literal",
+        }
 
     def visit_regex_literal(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Regex Literal", "node_class": "literal"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Regex Literal",
+            "node_class": "literal",
+        }
 
     def visit_unary_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Unary Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Unary Expression",
+            "node_class": "expression",
+        }
 
     def visit_parentheses_expression(self, node) -> dict[str, Any]:
         return {
@@ -981,52 +1044,108 @@ class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
         }
 
     def visit_set_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Set Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Set Expression",
+            "node_class": "expression",
+        }
 
     def visit_range_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Range Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Range Expression",
+            "node_class": "expression",
+        }
 
     def visit_function_call(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Function Call", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Function Call",
+            "node_class": "expression",
+        }
 
     def visit_array_access(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Array Access", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Array Access",
+            "node_class": "expression",
+        }
 
     def visit_member_access(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Member Access", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Member Access",
+            "node_class": "expression",
+        }
 
     def visit_condition(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Condition", "node_class": "condition"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Condition",
+            "node_class": "condition",
+        }
 
     def visit_for_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "For Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "For Expression",
+            "node_class": "expression",
+        }
 
     def visit_for_of_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "For-Of Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "For-Of Expression",
+            "node_class": "expression",
+        }
 
     def visit_at_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "At Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "At Expression",
+            "node_class": "expression",
+        }
 
     def visit_in_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "In Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "In Expression",
+            "node_class": "expression",
+        }
 
     def visit_of_expression(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Of Expression", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Of Expression",
+            "node_class": "expression",
+        }
 
     def visit_meta(self, node) -> dict[str, Any]:
         return {"id": self._get_node_id(), "label": "Meta", "node_class": "meta"}
 
     def visit_module_reference(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Module Reference", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Module Reference",
+            "node_class": "expression",
+        }
 
     def visit_dictionary_access(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Dictionary Access", "node_class": "expression"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Dictionary Access",
+            "node_class": "expression",
+        }
 
     def visit_comment(self, node) -> dict[str, Any]:
         return {"id": self._get_node_id(), "label": "Comment", "node_class": "comment"}
 
     def visit_comment_group(self, node) -> dict[str, Any]:
-        return {"id": self._get_node_id(), "label": "Comment Group", "node_class": "comment"}
+        return {
+            "id": self._get_node_id(),
+            "label": "Comment Group",
+            "node_class": "comment",
+        }
 
     def visit_defined_expression(self, node) -> dict[str, Any]:
         return {
@@ -1041,3 +1160,111 @@ class HtmlTreeGenerator(ASTVisitor[dict[str, Any]]):
             "label": "String Operator Expression",
             "node_class": "expression",
         }
+
+    def visit_extern_import(self, node) -> dict[str, Any]:
+        """Visit ExternImport node."""
+        return {
+            "id": self._get_node_id(),
+            "label": "Extern Import",
+            "node_class": "import",
+        }
+
+    def visit_extern_namespace(self, node) -> dict[str, Any]:
+        """Visit ExternNamespace node."""
+        return {
+            "id": self._get_node_id(),
+            "label": "Extern Namespace",
+            "node_class": "namespace",
+        }
+
+    def visit_extern_rule(self, node) -> dict[str, Any]:
+        """Visit ExternRule node."""
+        return {"id": self._get_node_id(), "label": "Extern Rule", "node_class": "rule"}
+
+    def visit_extern_rule_reference(self, node) -> dict[str, Any]:
+        """Visit ExternRuleReference node."""
+        return {
+            "id": self._get_node_id(),
+            "label": "Extern Rule Reference",
+            "node_class": "expression",
+        }
+
+    def visit_in_rule_pragma(self, node) -> dict[str, Any]:
+        """Visit InRulePragma node."""
+        return {
+            "id": self._get_node_id(),
+            "label": "In-Rule Pragma",
+            "node_class": "pragma",
+        }
+
+    def visit_pragma(self, node) -> dict[str, Any]:
+        """Visit Pragma node."""
+        return {"id": self._get_node_id(), "label": "Pragma", "node_class": "pragma"}
+
+    def visit_pragma_block(self, node) -> dict[str, Any]:
+        """Visit PragmaBlock node."""
+        return {
+            "id": self._get_node_id(),
+            "label": "Pragma Block",
+            "node_class": "pragma",
+        }
+
+
+# Alias for compatibility
+HTMLTreeGenerator = HtmlTreeGenerator
+
+
+def generate_html_tree(ast: YaraFile, title: str = "YARA AST") -> str:
+    """Generate HTML tree visualization from AST."""
+    gen = HTMLTreeGenerator()
+    return gen.generate_html(ast, None, title)
+
+
+def create_node_html(node: ASTNode) -> str:
+    """Create HTML for a single AST node."""
+    gen = HTMLTreeGenerator()
+    node_data = node.accept(gen)
+    
+    html = f'<div class="tree-node {node_data.get("node_class", "")}">'
+    html += f'<span class="node-label">{node_data.get("label", "")}</span>'
+    
+    if node_data.get("value"):
+        html += f'<span class="node-value">{node_data["value"]}</span>'
+    
+    if node_data.get("details"):
+        html += f'<div class="node-details">{node_data["details"]}</div>'
+    
+    html += '</div>'
+    return html
+
+
+def generate_ast_tree(ast: YaraFile) -> dict[str, Any]:
+    """Generate tree data structure from AST."""
+    gen = HTMLTreeGenerator()
+    return gen.visit(ast)
+
+
+def export_html_tree(ast: YaraFile, output_path: str | Path, 
+                     title: str = "YARA AST Visualization") -> None:
+    """Export HTML tree visualization to file."""
+    gen = HTMLTreeGenerator()
+    gen.generate_html(ast, str(output_path), title)
+
+
+# Add generate method to HTMLTreeGenerator for better compatibility
+def _generate_method(self, ast: YaraFile, title: str = "YARA AST", 
+                    expand_level: int = 2, show_attributes: bool = True,
+                    custom_css: str = "", custom_js: str = "") -> str:
+    """Generate HTML tree visualization."""
+    html = self.generate_html(ast, None, title)
+    
+    # Add custom CSS/JS if provided
+    if custom_css:
+        html = html.replace("</style>", f"{custom_css}\n</style>")
+    if custom_js:
+        html = html.replace("</script>", f"{custom_js}\n</script>")
+    
+    return html
+
+
+HTMLTreeGenerator.generate = _generate_method
