@@ -3,25 +3,68 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from typing import Any, TypeVar
-
-from yaraast.ast.base import *
-from yaraast.ast.comments import *
-from yaraast.ast.conditions import *
-from yaraast.ast.expressions import *
-from yaraast.ast.extern import *
-from yaraast.ast.meta import *
-from yaraast.ast.modifiers import *
-from yaraast.ast.modules import *
-from yaraast.ast.operators import *
-from yaraast.ast.pragmas import *
-from yaraast.ast.rules import *
-from yaraast.ast.strings import *
+from typing import Any, Generic, TypeVar, cast
 
 T = TypeVar("T")
 
+# Import specific classes from AST modules
+from yaraast.ast.base import ASTNode, YaraFile
+from yaraast.ast.comments import Comment, CommentGroup
+from yaraast.ast.conditions import (
+    AtExpression,
+    Condition,
+    ForExpression,
+    ForOfExpression,
+    InExpression,
+    OfExpression,
+)
+from yaraast.ast.expressions import (
+    ArrayAccess,
+    BinaryExpression,
+    BooleanLiteral,
+    DoubleLiteral,
+    Expression,
+    FunctionCall,
+    Identifier,
+    IntegerLiteral,
+    MemberAccess,
+    ParenthesesExpression,
+    RangeExpression,
+    RegexLiteral,
+    SetExpression,
+    StringCount,
+    StringIdentifier,
+    StringLength,
+    StringLiteral,
+    StringOffset,
+    UnaryExpression,
+)
+from yaraast.ast.extern import (
+    ExternImport,
+    ExternNamespace,
+    ExternRule,
+    ExternRuleReference,
+)
+from yaraast.ast.meta import Meta
+from yaraast.ast.modifiers import StringModifier
+from yaraast.ast.operators import DefinedExpression, StringOperatorExpression
+from yaraast.ast.pragmas import InRulePragma, Pragma, PragmaBlock
+from yaraast.ast.rules import Import, Include, Rule
+from yaraast.ast.simple_nodes import Tag
+from yaraast.ast.strings import (
+    HexAlternative,
+    HexByte,
+    HexJump,
+    HexString,
+    HexToken,
+    HexWildcard,
+    PlainString,
+    RegexString,
+    StringDefinition,
+)
 
-class ASTVisitor[T](ABC):
+
+class ASTVisitor(ABC, Generic[T]):
     """Base visitor class for traversing AST nodes."""
 
     def visit(self, node: ASTNode) -> T:
@@ -265,12 +308,20 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_yara_file(self, node: YaraFile) -> YaraFile:
         """Transform YaraFile node."""
+        from typing import cast
+
         imports = [self.visit(imp) for imp in node.imports]
         includes = [self.visit(inc) for inc in node.includes]
-        rules = [self.visit(rule) for rule in node.rules]
-        extern_rules = [self.visit(extern_rule) for extern_rule in node.extern_rules]
-        extern_imports = [self.visit(extern_import) for extern_import in node.extern_imports]
-        pragmas = [self.visit(pragma) for pragma in node.pragmas]
+        rules = cast("list[Rule]", [self.visit(rule) for rule in node.rules])
+        extern_rules = cast(
+            "list[ExternRule]",
+            [self.visit(extern_rule) for extern_rule in node.extern_rules],
+        )
+        extern_imports = cast(
+            "list[ExternImport]",
+            [self.visit(extern_import) for extern_import in node.extern_imports],
+        )
+        pragmas = cast("list[Pragma]", [self.visit(pragma) for pragma in node.pragmas])
         namespaces = [self.visit(namespace) for namespace in node.namespaces]
 
         return YaraFile(
@@ -281,7 +332,6 @@ class ASTTransformer(ASTVisitor[ASTNode]):
             extern_imports=extern_imports,
             pragmas=pragmas,
             namespaces=namespaces,
-            location=node.location,
         )
 
     def visit_import(self, node: Import) -> Import:
@@ -333,10 +383,13 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_hex_string(self, node: HexString) -> HexString:
         """Transform HexString node."""
-        tokens = [self.visit(token) for token in node.tokens]
-        modifiers = [self.visit(mod) for mod in node.modifiers]
+        tokens = [cast("HexToken", self.visit(token)) for token in node.tokens]
+        modifiers = [cast("StringModifier", self.visit(mod)) for mod in node.modifiers]
         return HexString(
-            identifier=node.identifier, tokens=tokens, modifiers=modifiers, location=node.location
+            identifier=node.identifier,
+            tokens=tokens,
+            modifiers=modifiers,
+            location=node.location,
         )
 
     def visit_regex_string(self, node: RegexString) -> RegexString:
@@ -367,11 +420,13 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_hex_jump(self, node: HexJump) -> HexJump:
         """Transform HexJump node."""
-        return HexJump(min_jump=node.min_jump, max_jump=node.max_jump, location=node.location)
+        return HexJump(min_jump=node.min_jump, max_jump=node.max_jump)
 
     def visit_hex_alternative(self, node: HexAlternative) -> HexAlternative:
         """Transform HexAlternative node."""
-        alternatives = [[self.visit(token) for token in alt] for alt in node.alternatives]
+        alternatives = [
+            [self.visit(token) for token in alt] for alt in node.alternatives
+        ]
         return HexAlternative(alternatives=alternatives, location=node.location)
 
     def visit_hex_nibble(self, node) -> Any:
@@ -386,63 +441,72 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_identifier(self, node: Identifier) -> Identifier:
         """Transform Identifier node."""
-        return Identifier(name=node.name, location=node.location)
+        # expressions.Identifier doesn't accept location
+        from yaraast.ast.expressions import Identifier as ExprIdentifier
+
+        return ExprIdentifier(name=node.name)
 
     def visit_string_identifier(self, node: StringIdentifier) -> StringIdentifier:
         """Transform StringIdentifier node."""
-        return StringIdentifier(name=node.name, location=node.location)
+        # StringIdentifier doesn't accept location parameter
+        return StringIdentifier(name=node.name)
 
     def visit_string_count(self, node: StringCount) -> StringCount:
         """Transform StringCount node."""
-        return StringCount(string_id=node.string_id, location=node.location)
+        return StringCount(string_id=node.string_id)
 
     def visit_string_offset(self, node: StringOffset) -> StringOffset:
         """Transform StringOffset node."""
-        index = self.visit(node.index) if node.index else None
-        return StringOffset(string_id=node.string_id, index=index, location=node.location)
+        index = cast("Expression", self.visit(node.index)) if node.index else None
+        return StringOffset(string_id=node.string_id, index=index)
 
     def visit_string_length(self, node: StringLength) -> StringLength:
         """Transform StringLength node."""
-        index = self.visit(node.index) if node.index else None
-        return StringLength(string_id=node.string_id, index=index, location=node.location)
+        index = cast("Expression", self.visit(node.index)) if node.index else None
+        return StringLength(string_id=node.string_id, index=index)
 
     def visit_integer_literal(self, node: IntegerLiteral) -> IntegerLiteral:
         """Transform IntegerLiteral node."""
-        return IntegerLiteral(value=node.value, location=node.location)
+        return IntegerLiteral(value=node.value)
 
     def visit_double_literal(self, node: DoubleLiteral) -> DoubleLiteral:
         """Transform DoubleLiteral node."""
-        return DoubleLiteral(value=node.value, location=node.location)
+        return DoubleLiteral(value=node.value)
 
     def visit_string_literal(self, node: StringLiteral) -> StringLiteral:
         """Transform StringLiteral node."""
-        return StringLiteral(value=node.value, location=node.location)
+        return StringLiteral(value=node.value)
 
     def visit_regex_literal(self, node: RegexLiteral) -> RegexLiteral:
         """Transform RegexLiteral node."""
-        return RegexLiteral(pattern=node.pattern, modifiers=node.modifiers, location=node.location)
+        return RegexLiteral(pattern=node.pattern, modifiers=node.modifiers)
 
     def visit_boolean_literal(self, node: BooleanLiteral) -> BooleanLiteral:
         """Transform BooleanLiteral node."""
-        return BooleanLiteral(value=node.value, location=node.location)
+        return BooleanLiteral(value=node.value)
 
     def visit_binary_expression(self, node: BinaryExpression) -> BinaryExpression:
         """Transform BinaryExpression node."""
-        left = self.visit(node.left)
-        right = self.visit(node.right)
-        return BinaryExpression(
-            left=left, operator=node.operator, right=right, location=node.location
-        )
+        from typing import cast
+
+        left = cast("Expression", self.visit(node.left))
+        right = cast("Expression", self.visit(node.right))
+        # BinaryExpression doesn't accept location parameter
+        return BinaryExpression(left=left, operator=node.operator, right=right)
 
     def visit_unary_expression(self, node: UnaryExpression) -> UnaryExpression:
         """Transform UnaryExpression node."""
-        operand = self.visit(node.operand)
-        return UnaryExpression(operator=node.operator, operand=operand, location=node.location)
+        operand = cast("Expression", self.visit(node.operand))
+        return UnaryExpression(operator=node.operator, operand=operand)
 
-    def visit_parentheses_expression(self, node: ParenthesesExpression) -> ParenthesesExpression:
+    def visit_parentheses_expression(
+        self, node: ParenthesesExpression
+    ) -> ParenthesesExpression:
         """Transform ParenthesesExpression node."""
-        expression = self.visit(node.expression)
-        return ParenthesesExpression(expression=expression, location=node.location)
+        from typing import cast
+
+        expression = cast("Expression", self.visit(node.expression))
+        return ParenthesesExpression(expression=expression)
 
     def visit_set_expression(self, node: SetExpression) -> SetExpression:
         """Transform SetExpression node."""
@@ -451,14 +515,18 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_range_expression(self, node: RangeExpression) -> RangeExpression:
         """Transform RangeExpression node."""
-        low = self.visit(node.low)
-        high = self.visit(node.high)
-        return RangeExpression(low=low, high=high, location=node.location)
+        from typing import cast
+
+        low = cast("Expression", self.visit(node.low))
+        high = cast("Expression", self.visit(node.high))
+        result = RangeExpression(low=low, high=high)
+        result.location = node.location
+        return result
 
     def visit_function_call(self, node: FunctionCall) -> FunctionCall:
         """Transform FunctionCall node."""
-        arguments = [self.visit(arg) for arg in node.arguments]
-        return FunctionCall(function=node.function, arguments=arguments, location=node.location)
+        arguments = [cast("Expression", self.visit(arg)) for arg in node.arguments]
+        return FunctionCall(function=node.function, arguments=arguments)
 
     def visit_array_access(self, node: ArrayAccess) -> ArrayAccess:
         """Transform ArrayAccess node."""
@@ -468,8 +536,11 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_member_access(self, node: MemberAccess) -> MemberAccess:
         """Transform MemberAccess node."""
-        object = self.visit(node.object)
-        return MemberAccess(object=object, member=node.member, location=node.location)
+        obj = self.visit(node.object)
+        # Ensure obj is an Expression
+        if not isinstance(obj, Expression):
+            obj = cast(Expression, obj)
+        return MemberAccess(object=obj, member=node.member)
 
     def visit_condition(self, node: Condition) -> Condition:
         """Transform Condition node."""
@@ -500,19 +571,19 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_at_expression(self, node: AtExpression) -> AtExpression:
         """Transform AtExpression node."""
-        offset = self.visit(node.offset)
-        return AtExpression(string_id=node.string_id, offset=offset, location=node.location)
+        offset = cast("Expression", self.visit(node.offset))
+        return AtExpression(string_id=node.string_id, offset=offset)
 
     def visit_in_expression(self, node: InExpression) -> InExpression:
         """Transform InExpression node."""
-        range = self.visit(node.range)
-        return InExpression(string_id=node.string_id, range=range, location=node.location)
+        range_expr = cast("Expression", self.visit(node.range))
+        return InExpression(string_id=node.string_id, range=range_expr)
 
     def visit_of_expression(self, node: OfExpression) -> OfExpression:
         """Transform OfExpression node."""
-        quantifier = self.visit(node.quantifier)
-        string_set = self.visit(node.string_set)
-        return OfExpression(quantifier=quantifier, string_set=string_set, location=node.location)
+        quantifier = cast("Expression", self.visit(node.quantifier))
+        string_set = cast("Expression", self.visit(node.string_set))
+        return OfExpression(quantifier=quantifier, string_set=string_set)
 
     def visit_meta(self, node: Meta) -> Meta:
         """Transform Meta node."""
@@ -564,7 +635,9 @@ class ASTTransformer(ASTVisitor[ASTNode]):
             location=node.location,
         )
 
-    def visit_extern_rule_reference(self, node: ExternRuleReference) -> ExternRuleReference:
+    def visit_extern_rule_reference(
+        self, node: ExternRuleReference
+    ) -> ExternRuleReference:
         """Transform ExternRuleReference node."""
         return ExternRuleReference(
             rule_name=node.rule_name, namespace=node.namespace, location=node.location
@@ -596,8 +669,8 @@ class ASTTransformer(ASTVisitor[ASTNode]):
 
     def visit_in_rule_pragma(self, node: InRulePragma) -> InRulePragma:
         """Transform InRulePragma node."""
-        pragma = self.visit(node.pragma)
-        return InRulePragma(pragma=pragma, position=node.position, location=node.location)
+        pragma = cast("Pragma", self.visit(node.pragma))
+        return InRulePragma(pragma=pragma, position=node.position)
 
     def visit_pragma_block(self, node: PragmaBlock) -> PragmaBlock:
         """Transform PragmaBlock node."""
